#include <iostream>
#include <strstream>
#include <gtk/gtk.h>
#include <stdlib.h>

using namespace std;

#include "GObject.h"
#include "DrawingArea.h"

gint configure_event(GtkWidget *wid, GdkEventConfigure *event, DrawingArea *dw)
{
	if(dw->getPixmap())
	{
		//avem pixmap
	}
	else
	{
		dw->setPixmap(gdk_pixmap_new(wid->window,wid->allocation.width,wid->allocation.height,-1));
		gdk_draw_rectangle(dw->getPixmap(),wid->style->white_gc,TRUE,0,0,wid->allocation.width,wid->allocation.height);
	}
	
	return TRUE;
}

gint expose_event(GtkWidget *wid, GdkEventExpose *event, DrawingArea *dw)
{
	gdk_draw_pixmap(wid->window, 
			wid->style->fg_gc[GTK_WIDGET_STATE(wid)],
			dw->getPixmap(),
			event->area.x,event->area.y,
			event->area.x,event->area.y,
			event->area.width,event->area.height);
	return FALSE;
}

GdkRectangle UpdateRectangle::getGdkRectangle()
{
	GdkRectangle gr;
	gr.x=_x1;
	gr.y=_y1;
	gr.width=_x2-_x1+1;
	gr.height=_y2-_y1+1;
	return gr;
}

GdkPixmap* DrawingArea::getPixmap()
{
	return _pixmap;
}

void DrawingArea::setPixmap(GdkPixmap *pixmap)
{
	_pixmap=pixmap;
}	

DrawingArea::DrawingArea(int width, int height, const string &namstr):
	GObject(namstr,(GtkWidget *)NULL)
{
	GtkWidget *drawingarea;
	drawingarea=gtk_drawing_area_new();
	gtk_drawing_area_size(GTK_DRAWING_AREA(drawingarea),width,height);
	setWidget(drawingarea);
	gtk_signal_connect(GTK_OBJECT(drawingarea),"expose_event",(GtkSignalFunc) expose_event, this);
	gtk_signal_connect(GTK_OBJECT(drawingarea),"configure_event",(GtkSignalFunc) configure_event, this);
	_pixmap=NULL;
}

void DrawingArea::line(int x1, int y1, int x2, int y2, int width, int red, int green, int blue)
{
	int xmin,xmax,ymin,ymax;
	
	if(x1<=x2)
	{
		xmin=x1;
		xmax=x2;
	}
	else
	{
		xmin=x2;
		xmax=x1;
	}
	
	if(y1<=y2)
	{
		ymin=y1;
		ymax=y2;
	}
	else
	{
		ymin=y2;
		ymax=y1;
	}
	
	xmin -= (width+1);
	xmax += (width+1);
	ymin -= (width+1);
	ymax += (width+1);
	
	if(xmin<_ur._x1) _ur._x1=xmin;
	if(xmax>_ur._x2) _ur._x2=xmax;
	
	if(ymin<_ur._y1) _ur._y1=ymin;
	if(ymax>_ur._y2) _ur._y2=ymax;
	
	GdkGC *gc=gdk_gc_new(_widget->window);
	GdkColor *color = (GdkColor *)malloc(sizeof(GdkColor));
	color->red=red*(65535/255);
	color->green=green*(65535/255);
	color->blue=blue*(65535/255);
	color->pixel=(gulong)(red*65536 + green*256 + blue);
	if(gdk_colormap_alloc_color(gtk_widget_get_colormap(_widget), color, false, true))
	{
		//cout<<"Color allocation OK"<<endl;
	}
	else
	{
		cout<<"Color allocation failed"<<endl;
	}

	gdk_gc_set_foreground(gc , color);
	gdk_gc_set_line_attributes(gc, width, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
	gdk_draw_line(_pixmap,
		      gc,
		      x1,y1,x2,y2);
	gdk_colormap_free_colors(gtk_widget_get_colormap(_widget), color, 1);
	gdk_gc_destroy(gc);
	free(color);
}

void DrawingArea::dashedLine(int x1, int y1, int x2, int y2, int width, int red, int green, int blue)
{
	int xmin,xmax,ymin,ymax;
	
	if(x1<x2)
	{
		xmin=x1;
		xmax=x2;
	}
	else
	{
		xmin=x2;
		xmax=x1;
	}
	
	if(y1<y2)
	{
		ymin=y1;
		ymax=y2;
	}
	else
	{
		ymin=y2;
		ymax=y1;
	}
	
	if(xmin<_ur._x1) _ur._x1=xmin;
	if(xmax>_ur._x2) _ur._x2=xmax;
	
	if(ymin<_ur._y1) _ur._y1=ymin;
	if(ymax>_ur._y2) _ur._y2=ymax;
	
	GdkGC *gc=gdk_gc_new(_widget->window);
	GdkColor *color = (GdkColor *)malloc(sizeof(GdkColor));
	color->red=red*(65535/255);
	color->green=green*(65535/255);
	color->blue=blue*(65535/255);
	color->pixel=(gulong)(red*65536 + green*256 + blue);
	if(gdk_colormap_alloc_color(gtk_widget_get_colormap(_widget), color, false, true))
	{
		//cout<<"Color allocation OK"<<endl;
	}
	else
	{
		cout<<"Color allocation failed"<<endl;
	}
	gdk_gc_set_foreground(gc , color);
	gdk_gc_set_line_attributes(gc, width, GDK_LINE_ON_OFF_DASH, GDK_CAP_ROUND, GDK_JOIN_ROUND);
	gdk_draw_line(_pixmap,
		      gc,
		      x1,y1,x2,y2);
	gdk_colormap_free_colors(gtk_widget_get_colormap(_widget), color, 1);
	gdk_gc_destroy(gc);
	free(color);
}

void DrawingArea::setBackground(char *name, int width, int height)
{
    GdkPixmap *pxm;
    GdkBitmap *mask;
    
    pxm = gdk_pixmap_create_from_xpm(_widget->window, &mask, &_widget->style->bg[GTK_STATE_NORMAL], name);
    gdk_draw_pixmap(_pixmap, _widget->style->bg_gc[GTK_WIDGET_STATE(_widget)],
		pxm, 0,0,0,0,width,height);
    gdk_pixmap_unref(pxm);
}

void DrawingArea::rectangle(int filled, int x, int y, int width, int height, int red, int green, int blue)
{
	int xmin,xmax,ymin,ymax;

	xmin=x;
	ymin=y;
	xmax=x+width;
	ymax=y+height;
		
	if(xmin<_ur._x1) _ur._x1=xmin;
	if(xmax>_ur._x2) _ur._x2=xmax;
	
	if(ymin<_ur._y1) _ur._y1=ymin;
	if(ymax>_ur._y2) _ur._y2=ymax;
	
	GdkGC *gc=gdk_gc_new(_widget->window);
	GdkColor *color = (GdkColor *)malloc(sizeof(GdkColor));
	color->red=red*(65535/255);
	color->green=green*(65535/255);
	color->blue=blue*(65535/255);
	color->pixel=(gulong)(red*65536 + green*256 + blue);
	if(gdk_colormap_alloc_color(gtk_widget_get_colormap(_widget), color, false, true))
	{
		//cout<<"Color allocation OK"<<endl;
	}
	else
	{
		cout<<"Color allocation failed"<<endl;
	}
	gdk_gc_set_foreground(gc, color);
	gdk_gc_set_background(gc, color);
	gdk_draw_rectangle(_pixmap,
			   gc,
			   filled,
			   x,y,width,height);
	gdk_colormap_free_colors(gtk_widget_get_colormap(_widget), color, 1);
	gdk_gc_destroy(gc);
	free(color);
}

void DrawingArea::arc(int filled, int x, int y, int width, int height, int angle1, int angle2, int red, int green, int blue)
{
	int xmin,xmax,ymin,ymax;
	
	xmin=x-width;
	ymin=y-height;
	xmax=x+width;
	ymax=y+height;
		
	if(xmin<_ur._x1) _ur._x1=xmin;
	if(xmax>_ur._x2) _ur._x2=xmax;
	
	if(ymin<_ur._y1) _ur._y1=ymin;
	if(ymax>_ur._y2) _ur._y2=ymax;

	GdkGC *gc=gdk_gc_new(_widget->window);
	GdkColor *color = (GdkColor *)malloc(sizeof(GdkColor));
	color->red=red*(65535/255);
	color->green=green*(65535/255);
	color->blue=blue*(65535/255);
	color->pixel=(gulong)(red*65536 + green*256 + blue);
	if(gdk_colormap_alloc_color(gtk_widget_get_colormap(_widget), color, false, true))
	{
		//cout<<"Color allocation OK"<<endl;
	}
	else
	{
		cout<<"Color allocation failed"<<endl;
	}
	gdk_gc_set_foreground(gc, color);
	gdk_gc_set_background(gc, color);
	gdk_draw_arc(_pixmap,
			   gc,
			   filled,
			   x,y,width,height,angle1,angle2);
	gdk_colormap_free_colors(gtk_widget_get_colormap(_widget), color, 1);
	gdk_gc_destroy(gc);
	free(color);
}

void DrawingArea::updateScreen()
{
	GdkRectangle gr=_ur.getGdkRectangle();
	gtk_widget_draw(_widget,&gr);
	_ur._x1=10000;
	_ur._y1=10000;
	_ur._y2=0;
	_ur._x2=0;
}

void DrawingArea::point(int x, int y, int red, int green, int blue)
{
	if(x < _ur._x1) _ur._x1=x;
	if(x > _ur._x2) _ur._x2=x;
	if(y < _ur._y1) _ur._y1=y;
	if(y > _ur._y2) _ur._y2=y;

	GdkGC *gc=gdk_gc_new(_widget->window);
	GdkColor *color = (GdkColor *)malloc(sizeof(GdkColor));
	color->red=red*(65535/255);
	color->green=green*(65535/255);
	color->blue=blue*(65535/255);
	color->pixel=(gulong)(red*65536 + green*256 + blue);
	if(gdk_colormap_alloc_color(gtk_widget_get_colormap(_widget), color, false, true))
	{
		//cout<<"Color allocation OK"<<endl;
	}
	else
	{
		cout<<"Color allocation failed"<<endl;
	}
	gdk_gc_set_foreground(gc, color);
	gdk_gc_set_background(gc, color);
	gdk_draw_line(_pixmap,
			  gc,
		      x,y,x,y);
	gdk_colormap_free_colors(gtk_widget_get_colormap(_widget), color, 1);
	gdk_gc_destroy(gc);
	free(color);
}

void DrawingArea::text(int x, int y, const char *text, int red, int green, int blue, GdkFont *font)
{
	GdkGC *gc=gdk_gc_new(_widget->window);
	GdkColor *color = (GdkColor *)malloc(sizeof(GdkColor));
	color->red=red*(65535/255);
	color->green=green*(65535/255);
	color->blue=blue*(65535/255);
	color->pixel=(gulong)(red*65536 + green*256 + blue);
	if(gdk_colormap_alloc_color(gtk_widget_get_colormap(_widget), color, false, true))
	{
		//cout<<"Color allocation OK"<<endl;
	}
	else
	{
		cout<<"Color allocation failed"<<endl;
	}
	gdk_gc_set_foreground(gc, color);
	int width=gdk_string_width(font, text);
	int height=gdk_string_height(font, text);
	rectangle(1, x-2, y-height-2, width+4, height+4, 0, 0, 0);
	rectangle(1, x-1, y-height-1, width+2, height+2, 255,255,0);
	gdk_draw_string(_pixmap, 
			font, 
			_widget->style->black_gc,
			x, y,
			text);
	gdk_colormap_free_colors(gtk_widget_get_colormap(_widget), color, 1);
	gdk_gc_destroy(gc);
	if(x<_ur._x1) _ur._x1=x;
	if(x+width>_ur._x2) _ur._x2=x+width;
	if(y-height<_ur._y1) _ur._y1=y-height;
	if(y>_ur._y2) _ur._y2=y;
}

void DrawingArea::circle(int filled, int x, int y, int r, int red, int green, int blue)
{
	arc(filled, x, y, r, r, 0, 24000, red, green, blue);
}

DrawingArea::~DrawingArea()
{
	gdk_pixmap_unref(_pixmap);
}
